// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <vector>

#include <aoc/common/Grid.hpp>
#include <aoc/framework/Solver.hpp>

class PrintQueue
    : public Solver
{
public:
    PrintQueue();
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    static constexpr int getNPages();
    static constexpr int getMaxPageNo();
    bool isProcessingOrderingRules_;
    int pageNoMapIndex_;
    std::vector<int> pageNoMap_;
    Grid<bool> orderingRules_;
    size_t getMapped(const int pageNo);
    void processOrderingRule(const std::string& line);
    void processUpdatePages(const std::string& line);
};
