// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <map>
#include <vector>

#include <aoc/common/Grid.hpp>
#include <aoc/common/Point2.hpp>
#include <aoc/framework/LinesSolver.hpp>

class ResonantCollinearity
    : public LinesSolver
{
public:
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void finish() override;
private:
    static constexpr char getEmptyChar();
    std::map<char, std::vector<Point2>> antennas_{};
    void addNewAntenna(Grid<bool>& antinodeGrid1, Grid<bool>& antinodeGrid2, Point2&& newPosition);
    void findNewAntinodes(Grid<bool>& antinodeGrid1, Grid<bool>& antinodeGrid2, Point2&& start,
        const Point2& increment);
    void addNewAntinode(Grid<bool>& antinodeGrid, Point2& newPosition, long long int& part);
};
