// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <aoc/common/Lines.hpp>
#include <aoc/extra/WarehouseBoxPusher.hpp>
#include <aoc/extra/WarehouseWideBoxPusher.hpp>
#include <aoc/framework/LinesSolver.hpp>

class WarehouseWoes
    : public LinesSolver
{
public:
    WarehouseWoes(const int inputFileNameSuffix = 0);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    bool isProcessingMap_;
    bool isSearchingStartPosition_;
    Lines lines2_;
    WarehouseBoxPusher warehouseBoxPusher1_;
    WarehouseWideBoxPusher warehouseBoxPusher2_;
    static constexpr char getRobotChar();
    static constexpr char getBoxChar();
    static constexpr char getLeftWideBoxChar();
    static constexpr char getRightWideBoxChar();
    static constexpr char getWallChar();
    static constexpr char getEmptyChar();
    void checkStartingPosition(const std::string& line);
    void addWarehouseMap2Line(const std::string& line);
    long long int calcAllGpsCoordinates(Lines& warehouseMap, const char boxChar);
    long long int calcGpsCoordinate(const size_t x, const size_t y);
};
