// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <memory>

#include <aoc/common/Point2.hpp>

// Inspired by https://stackoverflow.com/a/32279494
template <typename T>
class Grid
{
public:
    Grid(size_t nRows, size_t nColumns)
        : nRows_{ nRows }, nColumns_{ nColumns }, data_{ std::make_unique<T[]>(nRows * nColumns) }
    {
    }

    size_t getNRows() const
    {
        return nRows_;
    }

    size_t getNColumns() const
    {
        return nColumns_;
    }

    T& cell(size_t rowNo, size_t columnNo)
    {
        return data_[rowNo * nColumns_ + columnNo];
    }

    T& cell(Point2 position)
    {
        return data_[position.y * nColumns_ + position.x];
    }

    T* operator[](size_t rowNo)
    {
        return rowNo * nColumns_ + data_.get();
    }

    void fill(const T value)
    {
        for (size_t i = 0; i < nRows_ * nColumns_; i++)
        {
            data_[i] = value;
        }
    }
private:
    size_t nRows_;
    size_t nColumns_;
    std::unique_ptr<T[]> data_;
};
