// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/extra/WarehouseBoxPusher.hpp>

WarehouseBoxPusher::WarehouseBoxPusher(const char boxChar, const char wallChar, const char emptyChar)
    : boxChar_{ boxChar }, wallChar_{ wallChar }, emptyChar_{ emptyChar }
{
}

void WarehouseBoxPusher::processMovements(Lines& warehouseMap, const std::string& line)
{
    for (const char c : line)
    {
        processMovement(warehouseMap, Point2::getCardinalDirection(c));
    }
}

void WarehouseBoxPusher::processMovement(Lines& warehouseMap, const Point2& direction)
{
    const Point2 next{ robotPosition_ + direction };
    Point2 push{ next };
    while (warehouseMap.getCharAt(push) == boxChar_)
    {
        push += direction;
    }
    if (warehouseMap.getCharAt(push) != wallChar_)
    {
        if (push != next)
        {
            warehouseMap.setCharAt(push, boxChar_);
            warehouseMap.setCharAt(next, emptyChar_);
        }
        robotPosition_ = next;
    }
}

void WarehouseBoxPusher::setRobotPosition(const Point2& robotPosition)
{
    robotPosition_ = robotPosition;
}
