// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/framework/Solver.hpp>

#include <sstream>

Solver::Solver(const int inputFileNameSuffix)
    : inputFileNameSuffix_{ inputFileNameSuffix }, part1{ 0 }, part2{ 0 }
{
}

const std::string Solver::getInputFileName() const
{
    std::ostringstream stream;
    stream << clean(getPuzzleName());
    if (inputFileNameSuffix_ > 0)
    {
        stream << "_" << inputFileNameSuffix_;
    }
    stream << ".txt";
    return stream.str();
}

long long int Solver::getResultPart1() const
{
    return part1;
}

long long int Solver::getResultPart2() const
{
    return part2;
}

std::string Solver::clean(const std::string& original) const
{
    std::string cleaned{ original };
    size_t start_pos = 0;
    while ((start_pos = cleaned.find(" ", start_pos)) != std::string::npos)
    {
        cleaned.replace(start_pos, 1, "_");
    }
    return cleaned;
}
